﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using PLCCommLib;

namespace Sample
{
    /// <summary>マウスホイールで数値をINC/DECするテキストボックス(TextBoxの継承クラス)</summary>
    [Description("マウスホイールで数値をINC/DECするテキストボックスコントロールです。(TextBoxの継承クラス)")]
    [ToolboxBitmap(typeof(TextBox))]
    class NumericText : TextBox
    {

        #region Enum定義

        /// <summary>入力の型</summary>
        public enum Type : int
        {
            /// <summary>INC/DEC処理無</summary>
            NONE = -1,
            /// <summary>符号付整数</summary>
            INT = 0,
            /// <summary>符号付HEX</summary>
            INT_H

            // 符号無とか浮動小数にも対応させるならココに定義追加
            // 型別にMax/Minのフィールドとプロパティも追加しないと使いにくい
        }

        #endregion

        #region フィールド

        private Type _DatTyp;

        private long _iMin;
        private long _iMax;

        #endregion

        #region コンストラクタ

        /// <summary>既定のコンストラクタ</summary>
        /// <remarks>フィールド初期化</remarks>
        public NumericText()
            : base()
        {
            _DatTyp = Type.INT;

            _iMin = 0;
            _iMax = 32767;
        }

        #endregion

        #region プロパティ

        /// <summary>入力の型を設定・取得</summary>
        /// <value>入力の型</value>
        public Type _DataType
        {
            get { return _DatTyp; }
            set
            {
                if (_DatTyp != value)
                {
                    // NONEの場合は設定のみ
                    if (value == Type.NONE || _DatTyp == Type.NONE)
                    {
                        _DatTyp = value;
                        return;
                    }

                    // DEC⇔HEXで変換
                    if (_DatTyp == Type.INT)
                    {
                        long temp;
                        if (long.TryParse(this.Text, out temp))
                            this.Text = temp.ToString("X");
                    }
                    else if (_DatTyp == Type.INT_H)
                    {
                        long temp;
                        if (long.TryParse(this.Text, System.Globalization.NumberStyles.HexNumber, null, out temp))
                            this.Text = temp.ToString();
                    }

                    _DatTyp = value;
                }
            }
        }



        /// <summary>入力の最小値(符号付整数)を設定・取得</summary>
        /// <value>最小値(符号付整数)</value>
        [Description("入力の最小値(符号付整数)を示します。")]
        [DefaultValue(0)]
        [Category("カスタム")]
        public long _IMinimum
        {
            get { return _iMin; }
            set { _iMin = value; }
        }

        /// <summary>入力の最大値(符号付整数)を設定・取得</summary>
        /// <value>最大値(符号付整数)</value>
        [Description("入力の最小値(符号付整数)を示します。")]
        [DefaultValue(32767L)]
        [Category("カスタム")]
        public long _IMaximum
        {
            get { return _iMax; }
            set { _iMax = value; }
        }

        #endregion

        #region メソッド

        /// <summary>数値文字列のインクリメント処理</summary>
        /// <param name="div">変化量</param>
        internal virtual void OnIncrement(int div)
        {
            if (string.IsNullOrEmpty(this.SelectedText))
            {
                // ●TextBoxの文字列選択が無い状態 → 全体を数値としてINC/DEC処理
                int selLast = this.Text.Length - this.SelectionStart;
                long temp;
                long after;

                switch (_DataType)
                {
                    case Type.INT:
                        if (long.TryParse(this.Text, out temp))
                        {
                            after = temp + div;
                            if (after < _IMinimum) after = _IMinimum;
                            if (after > _IMaximum) after = _IMaximum;
                            if (after != temp)
                            {
                                this.Text = after.ToString();
                                if (this.Text.Length > selLast) this.SelectionStart = this.Text.Length - selLast;
                            }
                        }
                        break;

                    case Type.INT_H:
                        if (long.TryParse(this.Text, System.Globalization.NumberStyles.HexNumber, null, out temp))
                        {
                            after = temp + div;
                            if (after < _IMinimum) after = _IMinimum;
                            if (after > _IMaximum) after = _IMaximum;
                            if (after != temp)
                            {
                                this.Text = after.ToString("X");
                                if (this.Text.Length > selLast) this.SelectionStart = this.Text.Length - selLast;
                            }
                        }
                        break;
                }


            }
            else
            {
                // ●TextBoxの文字列選択状態 → 選択対象のみを数値としてINC/DEC処理
                int lastSelStart = this.SelectionStart;
                int lastSelLen = this.SelectionLength;
                string selText = this.SelectedText;

                // HEX、浮動小数の判別
                bool fHex = (_DataType == Type.INT_H) ? true : false;
                bool fFlt = false;

                // 符号と小数点等を除いた文字列を作る
                string num = "";
                string chk;
                for (int i = 0; i < selText.Length; i++)
                {
                    chk = selText.Substring(i, 1);
                    switch (chk)
                    {
                        case "+": case "-": case ".":
                            break;
                        case "E": case "e":
                            if (!fFlt) num += chk;
                            break;
                        default:
                            num += chk;
                            break;
                    }
                }

                // 4桁までしか対応しない
                long max;
                string fmt;
                if (fHex)
                {
                    switch (num.Length)
                    {
                        case 1: max = 0xF; fmt = "X1"; break;
                        case 2: max = 0xFF; fmt = "X2"; break;
                        case 3: max = 0xFFF; fmt = "X3"; break;
                        case 4: max = 0xFFFF; fmt = "X4"; break;
                        default: return;
                    }
                }
                else
                {
                    switch (num.Length)
                    {
                        case 1: max = 9; fmt = "D1"; break;
                        case 2: max = 99; fmt = "D2"; break;
                        case 3: max = 999; fmt = "D3"; break;
                        case 4: max = 9999; fmt = "D4"; break;
                        default: return;
                    }
                }

                // 文字→数値に変換
                ulong udat;
                long dat;
                if (fHex)
                {
                    if (!ulong.TryParse(num, System.Globalization.NumberStyles.HexNumber, null, out udat)) return;
                }
                else
                {
                    if (!ulong.TryParse(num, out udat)) return;
                }
                dat = DataConv.U2S(udat);

                // INC/DEC *最小⇔最大でローテーション
                string after;
                dat += div;
                if (dat < 0) dat += max + 1;
                if (dat > max) dat -= max + 1;
                after = dat.ToString(fmt);

                // 符号と小数点等を加味した文字列を作る
                string rep = "";
                int ii = 0;
                for (int i = 0; i < selText.Length; i++)
                {
                    chk = selText.Substring(i, 1);
                    switch (chk)
                    {
                        case "+": case "-": case ".":
                            rep += chk;
                            break;
                        case "E": case "e":
                            if (fFlt)
                            {
                                rep += chk;
                            }
                            else
                            {
                                rep += after.Substring(ii, 1);
                                ii++;
                            }
                            break;
                        default:
                            rep += after.Substring(ii, 1);
                            ii++;
                            break;
                    }
                }

                // 選択文字列を置換
                this.SelectedText = rep;
                this.SelectionStart = lastSelStart;
                this.SelectionLength = lastSelLen;


            }

        }

        #endregion

        #region メソッド(override)

        /// <summary>ホイールイベント処理メソッド >>> INC/DEC処理</summary>
        protected override void OnMouseWheel(MouseEventArgs e)
        {
            base.OnMouseWheel(e);
            if (_DataType == Type.NONE || string.IsNullOrEmpty(this.Text)) return;

            int div = e.Delta / SystemInformation.MouseWheelScrollDelta;    // ← 通常±3の範囲になるハズ
            OnIncrement(div);

        }

        /// <summary>キーダウン イベント処理メソッド >>> Ctrl+Shift+Up/DownでINC/DEC処理</summary>
        protected override void OnKeyDown(KeyEventArgs e)
        {
            if (_DataType == Type.NONE || string.IsNullOrEmpty(this.Text))
            {
                base.OnKeyDown(e);
                return;
            }

            if (e.Control && e.Shift)
            {
                if (e.KeyCode == Keys.Up)
                {
                    OnIncrement(1);
                    e.Handled = true;
                }
                else if (e.KeyCode == Keys.Down)
                {
                    OnIncrement(-1);
                    e.Handled = true;
                }
                else
                {
                    base.OnKeyDown(e);
                }
            }
            else
            {
                base.OnKeyDown(e);
            }

        }

        #endregion

    }
}
