﻿Imports System.ComponentModel

''' <summary>マウスホイールで数値をINC/DECするテキストボックス(TextBoxの継承クラス)</summary>
<Description("マウスホイールで数値をINC/DECするテキストボックスコントロールです。(TextBoxの継承クラス)")> _
<ToolboxBitmap(GetType(TextBox))> _
Public Class NumericText
    Inherits TextBox

#Region "Enum定義"

    ''' <summary>入力の型</summary>
    Public Enum Type As Integer
        ''' <summary>INC/DEC処理無</summary>
        NONE = -1
        ''' <summary>符号付整数</summary>
        INT = 0
        ''' <summary>符号付HEX</summary>
        INT_H

        ' 符号無とか浮動小数にも対応させるならココに定義追加
        ' 型別にMax/Minのフィールドとプロパティも追加しないと使いにくい
    End Enum

#End Region

#Region "フィールド"

    Private _DatTyp As Type

    Private _iMin As Long
    Private _iMax As Long

#End Region

#Region "コンストラクタ"

    ''' <summary>既定のコンストラクタ</summary>
    ''' <remarks>フィールド初期化</remarks>
    Public Sub New()
        MyBase.New()

        _DatTyp = Type.INT

        _iMin = 0
        _iMax = 32767
    End Sub

#End Region

#Region "プロパティ"

    ''' <summary>入力の型を設定・取得</summary>
    ''' <value>入力の型</value>
    ''' <returns>入力の型</returns>
    <Description("入力の型を示します。")> _
    <DefaultValue(GetType(Type), "INT")> _
    <Category("カスタム")> _
    Public Property _DataType() As Type
        Get
            Return _DatTyp
        End Get
        Set(ByVal value As Type)
            If _DatTyp <> value Then
                ' NONEの場合は設定のみ
                If value = Type.NONE OrElse _DatTyp = Type.NONE Then
                    _DatTyp = value
                    Return
                End If

                ' DEC⇔HEXで変換
                If _DatTyp = Type.INT Then
                    Dim temp As Long
                    If Long.TryParse(Me.Text, temp) Then
                        Me.Text = temp.ToString("X")
                    End If

                ElseIf _DatTyp = Type.INT_H Then
                    Dim temp As Long
                    If Long.TryParse(Me.Text, Globalization.NumberStyles.HexNumber, Nothing, temp) Then
                        Me.Text = temp.ToString()
                    End If

                End If

                _DatTyp = value
            End If
        End Set
    End Property



    ''' <summary>入力の最小値(符号付整数)を設定・取得</summary>
    ''' <value>最小値(符号付整数)</value>
    ''' <returns>最小値(符号付整数)</returns>
    <Description("入力の最小値(符号付整数)を示します。")> _
    <DefaultValue(0)> _
    <Category("カスタム")> _
    Public Property _IMinimum() As Long
        Get
            Return _iMin
        End Get
        Set(ByVal value As Long)
            _iMin = value
        End Set
    End Property

    ''' <summary>入力の最大値(符号付整数)を設定・取得</summary>
    ''' <value>最大値(符号付整数)</value>
    ''' <returns>最大値(符号付整数)</returns>
    <Description("入力の最小値(符号付整数)を示します。")> _
    <DefaultValue(32767L)> _
    <Category("カスタム")> _
    Public Property _IMaximum() As Long
        Get
            Return _iMax
        End Get
        Set(ByVal value As Long)
            _iMax = value
        End Set
    End Property

#End Region

#Region "メソッド"

    ''' <summary>数値文字列のインクリメント処理</summary>
    ''' <param name="div">変化量</param>
    Friend Overridable Sub OnIncrement(ByVal div As Integer)

        If String.IsNullOrEmpty(Me.SelectedText) Then
            ' ●TextBoxの文字列選択が無い状態 → 全体を数値としてINC/DEC処理
            Dim selLast As Integer = Me.Text.Length - Me.SelectionStart
            Dim temp As Long
            Dim after As Long

            Select Case _DataType
                Case Type.INT
                    If Long.TryParse(Me.Text, temp) Then
                        after = temp + div
                        If after < _IMinimum Then after = _IMinimum
                        If after > _IMaximum Then after = _IMaximum
                        If after <> temp Then
                            Me.Text = after.ToString()
                            If Me.Text.Length > selLast Then Me.SelectionStart = Me.Text.Length - selLast
                        End If
                    End If

                Case Type.INT_H
                    If Long.TryParse(Me.Text, Globalization.NumberStyles.HexNumber, Nothing, temp) Then
                        after = temp + div
                        If after < _IMinimum Then after = _IMinimum
                        If after > _IMaximum Then after = _IMaximum
                        If after <> temp Then
                            Me.Text = after.ToString("X")
                            If Me.Text.Length > selLast Then Me.SelectionStart = Me.Text.Length - selLast
                        End If
                    End If

            End Select


        Else
            ' ●TextBoxの文字列選択状態 → 選択対象のみを数値としてINC/DEC処理
            Dim lastSelStart As Integer = Me.SelectionStart
            Dim lastSelLen As Integer = Me.SelectionLength
            Dim selText As String = Me.SelectedText

            ' HEX、浮動小数の判別
            Dim fHex As Boolean = If(_DataType = Type.INT_H, True, False)
            Dim fFlt As Boolean = False

            ' 符号と小数点等を除いた文字列を作る
            Dim num As String = ""
            Dim chk As String
            For i = 0 To selText.Length - 1
                chk = selText.Substring(i, 1)
                Select Case chk
                    Case "+", "-", "."
                    Case "E", "e"
                        If fFlt = False Then num = num & chk
                    Case Else
                        num = num & chk
                End Select
            Next

            ' 4桁までしか対応しない
            Dim max As Long
            Dim fmt As String
            If fHex Then
                Select Case num.Length
                    Case 1 : max = &HF : fmt = "X1"
                    Case 2 : max = &HFF : fmt = "X2"
                    Case 3 : max = &HFFF : fmt = "X3"
                    Case 4 : max = &HFFFF : fmt = "X4"
                    Case Else : Return
                End Select
            Else
                Select Case num.Length
                    Case 1 : max = 9 : fmt = "D1"
                    Case 2 : max = 99 : fmt = "D2"
                    Case 3 : max = 999 : fmt = "D3"
                    Case 4 : max = 9999 : fmt = "D4"
                    Case Else : Return
                End Select
            End If

            ' 文字→数値に変換
            Dim udat As ULong
            Dim dat As Long
            If fHex Then
                If ULong.TryParse(num, Globalization.NumberStyles.HexNumber, Nothing, udat) = False Then Return
            Else
                If ULong.TryParse(num, udat) = False Then Return
            End If
            dat = DataConv.U2S(udat)

            ' INC/DEC *最小⇔最大でローテーション
            Dim after As String
            dat += div
            If dat < 0 Then dat += max + 1
            If dat > max Then dat -= max + 1
            after = dat.ToString(fmt)

            ' 符号と小数点等を加味した文字列を作る
            Dim rep As String = ""
            Dim ii As Integer = 0
            For i = 0 To selText.Length - 1
                chk = selText.Substring(i, 1)
                Select Case chk
                    Case "+", "-", "." : rep = rep & chk
                    Case "E", "e"
                        If fFlt Then
                            rep = rep & chk
                        Else
                            rep = rep & after.Substring(ii, 1)
                            ii = ii + 1
                        End If
                    Case Else
                        rep = rep & after.Substring(ii, 1)
                        ii = ii + 1
                End Select
            Next

            ' 選択文字列を置換
            Me.SelectedText = rep
            Me.SelectionStart = lastSelStart
            Me.SelectionLength = lastSelLen


        End If

    End Sub

#End Region

#Region "メソッド(Overrides)"

    ''' <summary>ホイールイベント処理メソッド >>> INC/DEC処理</summary>
    Protected Overrides Sub OnMouseWheel(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseWheel(e)
        If _DataType = Type.NONE OrElse String.IsNullOrEmpty(Me.Text) Then Return

        Dim div As Integer = e.Delta \ SystemInformation.MouseWheelScrollDelta ' ← 通常±3の範囲になるハズ
        OnIncrement(div)

    End Sub

    ''' <summary>キーダウン イベント処理メソッド >>> Ctrl+Shift+Up/DownでINC/DEC処理</summary>
    Protected Overrides Sub OnKeyDown(ByVal e As System.Windows.Forms.KeyEventArgs)

        If _DataType = Type.NONE OrElse String.IsNullOrEmpty(Me.Text) Then
            MyBase.OnKeyDown(e)
            Return
        End If

        If e.Control AndAlso e.Shift Then
            If e.KeyCode = Keys.Up Then
                OnIncrement(1)
                e.Handled = True
            ElseIf e.KeyCode = Keys.Down Then
                OnIncrement(-1)
                e.Handled = True
            Else
                MyBase.OnKeyDown(e)
            End If
        Else
            MyBase.OnKeyDown(e)
        End If

    End Sub

#End Region

End Class
